#!/usr/bin/env python


import sys
import os
import subprocess


if len(sys.argv) >= 2:
	bact_id = sys.argv[1]


ini_path = 'nominal.ini'
if len(sys.argv) >= 3:
    ini_path = sys.argv[2]


run_identifier = 'A'
if len(sys.argv) >= 4:
    run_identifier = sys.argv[3]

graphml_file = bact_id+run_identifier+'.G2.graphml'


# This is used to start the pipeline in the middle
st_point = 0
if len(sys.argv) >= 5:
	st_point = int(sys.argv[4])


# This is used to stop the pipeline in the middle
end_point = 20
if len(sys.argv) >= 6:
    end_point = int(sys.argv[5])



base_path = './'

if st_point <= 1 and end_point >= 1:
    draft_path_cmd = 'get_draft_path_norevcomp.py '+base_path+' '+ bact_id+' '+graphml_file
    print '1: '+draft_path_cmd
    subprocess.check_output(draft_path_cmd,cwd=base_path, shell=True)


if st_point <= 2 and end_point >= 2:
    draft_assembly_cmd = 'draft_assembly --db '+bact_id+' --las '+bact_id+'.las --prefix '+bact_id+' --config '+ini_path+' --out '+bact_id+'.draft'
    print '2: '+draft_assembly_cmd
    subprocess.check_output(draft_assembly_cmd,cwd=base_path, shell=True)
  

if st_point <= 3 and end_point >= 3:
    corr_head_cmd = 'correct_head.py '+bact_id+'.draft.fasta '+bact_id+'.draft.pb.fasta draft_map.txt'
    print '3: '+corr_head_cmd
    subprocess.check_output(corr_head_cmd,cwd=base_path, shell=True)


if st_point <= 4 and end_point >= 4:
    subprocess.call("rm -f draft.db",shell=True,cwd=base_path)
    fasta2DB_cmd = "fasta2DB draft "+base_path+bact_id+'.draft.pb.fasta'
    print '4: '+fasta2DB_cmd
    subprocess.check_output(fasta2DB_cmd.split(),cwd=base_path)

if st_point <= 5 and end_point >= 5:
    subprocess.call("rm -f draft.*.las",shell=True,cwd=base_path)
    mapper_cmd = "HPCmapper draft "+bact_id
    print '5: '+mapper_cmd
    subprocess.call(mapper_cmd.split(),stdout=open(base_path+'draft_consensus.sh','w') , cwd=base_path)



if st_point <= 6 and end_point >= 6:
    # modify_cmd = """awk '{gsub("daligner -A -k20 -h50 -e.85","daligner -A",$0); print $0}' draft_consensus.sh"""
    modify_cmd = ['awk','{gsub("daligner -A -k20 -h50 -e.85","daligner -A",$0); print $0}','draft_consensus.sh']
    print '6: '+"""awk '{gsub("daligner -A -k20 -h50 -e.85","daligner -A",$0); print $0}' draft_consensus.sh"""
    subprocess.call(modify_cmd,stdout=open(base_path+'draft_consensus2.sh','w') , cwd=base_path)



if st_point <= 7 and end_point >= 7:
    mapper_shell_cmd = "csh -v draft_consensus2.sh"
    print '7: '+mapper_shell_cmd
    subprocess.check_output(mapper_shell_cmd.split(), cwd=base_path)


if st_point <= 8 and end_point >= 8:
    # remove_cmd = 'rm -f nonrevcompdraft.'+bact_id+'.*.las'
    # subprocess.call(remove_cmd,shell=True,cwd=base_path)
    LAmerge_cmd = "LAmerge draft."+bact_id+".las "+'draft.'+bact_id+'.[0-9].las'
    print '8: '+LAmerge_cmd
    subprocess.check_output(LAmerge_cmd,cwd=base_path,shell=True)


if st_point <= 9 and end_point >= 9:
    consensus_cmd = 'consensus draft '+bact_id+' draft.'+bact_id+'.las '+bact_id+'.norevcomp_consensus.fasta '+ini_path
    print '9: '+consensus_cmd
    subprocess.check_output(consensus_cmd,cwd=base_path,shell=True)
    







#!/usr/bin/env python

import sys 
import os



def run(inputfile,outputfile):


	nodes = {}
	arcs = {}

	with open(inputfile) as file:

	    for line in file:

			line_str = line[:-1]
			split_str = line_str.split(' ')

			node0 = int(split_str[0])
			node1 = int(split_str[1])

			# print node0,node1

			nodes[node0] = 1
			nodes[node1] = 1

			if node0 < node1:
				arcs[tuple([node0,node1])] = 1
			else:
				arcs[tuple([node1,node0])] = 1



	with open(outputfile, 'w') as fout:

		for node in nodes:
			fout.write("NODE "+str(node)+' 0 0 0 0 0\n')
			fout.write('AAA\n')
			fout.write('AAA\n')
			# print "NODE "+str(node)

		for arc in arcs:
			fout.write("ARC "+str(arc[0])+' '+str(arc[1])+' 0\n')

		



def main():

    run(sys.argv[1],sys.argv[2])
    return


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import sys
import os
import subprocess
from parse_read import *
from parse_alignment import *

filename = sys.argv[1]
readarg = sys.argv[2]


stream = subprocess.Popen(["LAshow", filename , filename ,readarg],
                                  stdout=subprocess.PIPE, bufsize=1)

alignments = parse_alignment(stream.stdout) # generator

for alignment in alignments:
    print alignment



#!/usr/bin/env python

import sys
import os
import subprocess
from parse_read import *
import numpy as np
import networkx as nx
import itertools
from pbcore.io import FastaIO




def rev_node(node):
    node_id = node.split('_')[0]
    return node_id + '_' + str(1-int(node.split('_')[1]))



def merge_nodes(g,in_node,out_node):

    weight = str(g.edge[in_node][out_node]['length'])

    if 'path' in g.node[in_node]:
        path1 = g.node[in_node]['path']
        weightspath1 = g.node[in_node]['weightspath']
    else:
        path1 = in_node
        weightspath1 = ''

    if 'path' in g.node[out_node]:
        path2 = g.node[out_node]['path']
        weightspath2 = ';' + g.node[out_node]['weightspath']
    else:
        path2 = out_node
        weightspath2 = ''

    g.node[in_node]['path'] = path1 + ';' + path2

    if weightspath1 == '':
        g.node[in_node]['weightspath'] = weight + weightspath2
    else:
        g.node[in_node]['weightspath'] = weightspath1 + ';' + weight + weightspath2

    for nodeB in g.successors(out_node):
        g.add_edge(in_node,nodeB,length=g.edge[out_node][nodeB]['length'])


    g.node[in_node]['cut_end'] = g.node[out_node]['cut_end']
    g.remove_node(out_node)



filedir = sys.argv[1]
filename = sys.argv[2]
graphml_path = sys.argv[3]

in_graph = nx.read_graphml(graphml_path)

# debug output
#for node in in_graph.nodes():
#    print node

#for edge in in_graph.edges():
#    print len(in_graph.edge[edge[0]][edge[1]])

reads = sorted(list(set([int(x.split("_")[0].lstrip("B")) for x in in_graph.nodes()])))

dbshow_reads = ' '.join([str(x+1) for x in reads])

DBshow_cmd = "DBshow "+ filedir+'/'+ filename+' '+dbshow_reads
stream = subprocess.Popen(DBshow_cmd.split(),
                                  stdout=subprocess.PIPE,bufsize=1)
reads_queried = parse_read(stream.stdout)
read_dict = {}
for read_id,read in itertools.izip(reads,reads_queried):
    rdlen = len(read[1])
#     print read
    read_dict[read_id] = read

# to simulate reads

# read_dict = {}
# for vertex in in_graph.nodes():
#     read_dict[int(vertex.split('_')[0])] = ['A','A'*50000]


complement = {'A':'T','C': 'G','T':'A', 'G':'C','a':'t','t':'a','c':'g','g':'c'}

# out_graphml_name = 'test.graphml'
out_graphml_name = filedir + '/' + filename +'_draft.graphml'

# outfile = 'test.edges.list'
outfile = filedir + '/' + filename + ".edges.list"


rev_comp_contig = True

out_graph = in_graph.copy()


# first we add some info to the graph for the cutting of contigs
for vert in out_graph.nodes():

    vert_id, vert_or = vert.split("_")
    vert_id = vert_id.lstrip("B")

    vert_len = len(read_dict[int(vert_id)][1])

    out_graph.node[vert]['cut_start'] = 0
    out_graph.node[vert]['cut_end'] = vert_len


    # SHOULD THIS USE THE RAW MATCHES?

    if out_graph.in_degree(vert) > 1:
        if vert_or == '0':
            out_graph.node[vert]['cut_start'] = max([out_graph.edge[x][vert]['read_b_match_start'] for x in out_graph.predecessors(vert)])
        else:
            out_graph.node[vert]['cut_start'] = vert_len - min([out_graph.edge[vert_id+'_0'][x]['read_a_match_start'] for x in out_graph.successors(vert_id+'_0')])

    if out_graph.out_degree(vert) > 1:

        if vert_or == '0':
            out_graph.node[vert]['cut_end'] = min([out_graph.edge[vert][x]['read_a_match_start'] for x in out_graph.successors(vert)])
        else:
            out_graph.node[vert]['cut_end'] = vert_len - max([out_graph.edge[x][vert_id+'_0']['read_b_match_start'] for x in out_graph.predecessors(vert_id+'_0')])




# next we merge the nodes in out_graph to form the contigs

nodes_to_merge = [x for x in out_graph.nodes() if out_graph.in_degree(x) == 1 and out_graph.out_degree(out_graph.predecessors(x)[0]) == 1]



# print len(read_dict[41260][1])
# print len(read_dict[4697][1])


while nodes_to_merge:

    cur_node = nodes_to_merge[0]

    prev_node = out_graph.predecessors(cur_node)[0]


    if prev_node != cur_node:
        merge_nodes(out_graph,prev_node,cur_node)
    else:
        out_graph.node[cur_node]['path'] = out_graph.node[cur_node]['path'] + ';' + cur_node
        out_graph.node[cur_node]['weightspath'] = out_graph.node[cur_node]['weightspath'] + ';' + str(out_graph.edge[prev_node][cur_node]['length'])
        out_graph.node[cur_node]['cut_end'] = len(read_dict[int(cur_node.split('_')[0])][1])


    nodes_to_merge.pop(0)

    # print len(nodes_to_merge)






# next we print the contigs out to the .edges.list file
contig_no = 0
# print "Writing out_graph with "+str(len(out_graph.nodes()))+" contigs/nodes"


# we keep track of the already printed nodes so that reverse complement pairs are printed together
# we don't add to printed_nodes the "border" nodes so that we still have a partition of the nodes into contigs
# printed_nodes = set()
printed_nodes = {}

# debug output

# for node in out_graph.nodes():
#     print node
#     print out_graph.node[node]


# for edge in out_graph.edges():
#     print edge
#     print out_graph.edge[edge[0]][edge[1]]




with open(outfile, 'w') as f:

    for vertex in out_graph.nodes():

        if rev_node(vertex) in printed_nodes:
            out_graph.node[vertex]['contig_id'] = printed_nodes[rev_node(vertex)] + 1

            continue

        # single-node contig
        if 'path' not in out_graph.node[vertex]:

            out_graph.node[vertex]['contig_id'] = contig_no + 1
            f.write('>Unitig%d\n'%(contig_no))

            # printed_nodes = printed_nodes | set([vertex])
            printed_nodes[vertex] = contig_no

            contig_no += 1

            # we repeat the same node twice so that the line is easily distinguishable (6 numbers)
            f.write('O %s %s %s %s %d %d\n'%(vertex.split('_')[0].lstrip('B'), vertex.split('_')[1]  , vertex.split('_')[0].lstrip('B'),
                vertex.split('_')[1], out_graph.node[vertex]['cut_start'], out_graph.node[vertex]['cut_end']) )




            f.write('>Unitig%d\n'%(contig_no))
            contig_no += 1

            vertex_rc = rev_node(vertex)
            f.write('O %s %s %s %s %d %d\n'%(vertex_rc.split('_')[0].lstrip('B'), vertex_rc.split('_')[1]  , vertex_rc.split('_')[0].lstrip('B'),
                vertex_rc.split('_')[1], out_graph.node[vertex_rc]['cut_start'], out_graph.node[vertex_rc]['cut_end']) )
            continue




        node_list = out_graph.node[vertex]['path'].split(';')
        weights_list = out_graph.node[vertex]['weightspath'].split(';')




        # double-node contig
        if out_graph.in_degree(vertex) != 1 and out_graph.out_degree(vertex) != 1 and len(node_list) == 2:

            out_graph.node[vertex]['contig_id'] = contig_no
            f.write('>Unitig%d\n'%(contig_no))

            # printed_nodes = printed_nodes | set(node_list)
            printed_nodes[node_list[0]] = contig_no
            printed_nodes[node_list[1]] = contig_no

            contig_no += 1

            nodeA = node_list[0]
            nodeB = node_list[1]
            f.write('D %s %s %s %s %s %d %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[0], out_graph.node[vertex]['cut_start'], out_graph.node[vertex]['cut_end']) )



            f.write('>Unitig%d\n'%(contig_no))
            contig_no += 1

            nodeA = rev_node(node_list[1])
            nodeB = rev_node(node_list[0])
            f.write('D %s %s %s %s %s %d %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[0], len(read_dict[int(nodeA.split('_')[0])][1]) - out_graph.node[vertex]['cut_end'], len(read_dict[int(nodeB.split('_')[0])][1]) - out_graph.node[vertex]['cut_start'] ) )

            continue





        # print out_graph.node[vertex]['path']
        # print node_list
        # print out_graph.node[vertex]['weightspath']
        # print weights_list

        # print len(node_list),len(weights_list)

        if len(node_list) != len(weights_list)+1:
            print 'Something went wrong with contig '+str(contig_no)
            continue

        # printed_nodes = printed_nodes | set(node_list)
        for curnode in node_list:
            printed_nodes[curnode] = contig_no


        # print 'Unitig ' +str(contig_no) + ' ('+str(len(node_list))+' nodes)'

        out_graph.node[vertex]['contig_id'] = contig_no
        f.write('>Unitig%d\n'%(contig_no))
        contig_no += 1


        # prev_vert = out_graph.node[node_list[0]]['prev_node']
        # if prev_vert != '':

        if out_graph.in_degree(vertex) == 1 and out_graph.predecessors(vertex)[0] != vertex:

            prev_contig = out_graph.predecessors(vertex)[0]
            cut_start = out_graph.node[prev_contig]['cut_end']
            if out_graph.node[prev_contig].has_key('path'):
                nodeA = out_graph.node[prev_contig]['path'].split(';')[-1]
            else:
                nodeA = prev_contig

            nodeB = node_list[0]
            f.write('S %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'),
                nodeB.split('_')[1], out_graph.edge[prev_contig][vertex]['length'], cut_start) )


            if len(node_list) > 2:

                nodeA = node_list[0]
                nodeB = node_list[1]
                f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'),nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[0]) )


        else:
            nodeA = node_list[0]
            nodeB = node_list[1]
            f.write('S %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[0], out_graph.node[vertex]['cut_start']) )

        for i in range(1,len(weights_list)-1):
            nodeA = node_list[i]
            nodeB = node_list[i+1]
            f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[i]) )



        if out_graph.out_degree(vertex) == 1 and out_graph.successors(vertex)[0] != vertex:

            if len(node_list) > 2:

                nodeA = node_list[len(weights_list)-1]
                nodeB = node_list[len(weights_list)]
                f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[-1]) )

            next_contig = out_graph.successors(vertex)[0]
            # we end this contig where the next one begins
            cut_end = out_graph.node[next_contig]['cut_start']

            nodeA = node_list[len(weights_list)]
            if out_graph.node[next_contig].has_key('path'):
                nodeB = out_graph.node[next_contig]['path'].split(';')[0]
            else:
                nodeB = next_contig

            f.write('E %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'),nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'),
                nodeB.split('_')[1], out_graph.edge[vertex][next_contig]['length'], cut_end) )



        else:

            nodeA = node_list[len(weights_list)-1]
            nodeB = node_list[len(weights_list)]
            f.write('E %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'),nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[-1], out_graph.node[vertex]['cut_end']) )




        # if we want reverse complement contigs, we print them next to each other

        if rev_comp_contig == False:
            continue


        f.write('>Unitig%d\n'%(contig_no))
        contig_no += 1



        if out_graph.out_degree(vertex) == 1 and out_graph.successors(vertex)[0] != vertex:

            next_contig = out_graph.successors(vertex)[0]

            nodeB = rev_node(node_list[len(weights_list)])
            if out_graph.node[next_contig].has_key('path'):
                nodeA = rev_node(out_graph.node[next_contig]['path'].split(';')[0])
            else:
                nodeA = rev_node(next_contig)

            # we start this contig where the previous (rc: next) one ended
            cut_start = len(read_dict[int(nodeA.split('_')[0])][1]) - out_graph.node[next_contig]['cut_start']

            f.write('S %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'),
                nodeB.split('_')[1], out_graph.edge[vertex][next_contig]['length'], cut_start) )


            if len(node_list) > 2:

                nodeA = rev_node(node_list[len(weights_list)])
                nodeB = rev_node(node_list[len(weights_list)-1])
                f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[-1]) )

        else:

            nodeA = rev_node(node_list[len(weights_list)])
            nodeB = rev_node(node_list[len(weights_list)-1])

            f.write('S %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[-1], len(read_dict[int(nodeA.split('_')[0])][1]) - out_graph.node[vertex]['cut_end']) )



        for i in range(len(weights_list)-1,1,-1):
            nodeA = rev_node(node_list[i])
            nodeB = rev_node(node_list[i-1])
            f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[i-1]) )

        if out_graph.in_degree(vertex) == 1 and out_graph.predecessors(vertex)[0] != vertex:

            if len(node_list) > 2:

                nodeA = rev_node(node_list[1])
                nodeB = rev_node(node_list[0])
                f.write('T %s %s %s %s %s\n'%(nodeA.split('_')[0].lstrip('B'), nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1], weights_list[0]) )

            prev_contig = out_graph.predecessors(vertex)[0]

            nodeA = rev_node(node_list[0])

            if out_graph.node[prev_contig].has_key('path'):
                nodeB = rev_node(out_graph.node[prev_contig]['path'].split(';')[-1])
            else:
                nodeB = rev_node(prev_contig)
            cut_end = len(read_dict[int(nodeB.split('_')[0])][1]) - out_graph.node[prev_contig]['cut_end']

            f.write('E %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'),nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'),
                nodeB.split('_')[1], out_graph.edge[prev_contig][vertex]['length'], cut_end) )

        else:
            nodeB = rev_node(node_list[0])
            nodeA = rev_node(node_list[1])
            f.write('E %s %s %s %s %s %d\n'%(nodeA.split('_')[0].lstrip('B'),nodeA.split('_')[1]  , nodeB.split('_')[0].lstrip('B'), nodeB.split('_')[1],
                weights_list[0], len(read_dict[int(nodeB.split('_')[0])][1]) - out_graph.node[vertex]['cut_start']) )




print "Number of contigs: "+str(contig_no)

nx.write_graphml(out_graph,out_graphml_name)








#!/usr/bin/env python
import sys
import os
import subprocess
from parse_read import *
import numpy as np
import networkx as nx
import itertools

NCTCname = sys.argv[1]
filename = '/data/pacbio_assembly/pb_data/NCTC/'+NCTCname+'/'+NCTCname
graphml_path = sys.argv[2]

in_graph = nx.read_graphml(graphml_path)

reads = sorted(list(set([int(x.split("_")[0].lstrip("B")) for x in in_graph.nodes()])))

dbshow_reads = ' '.join([str(x+1) for x in reads])

DBshow_cmd = "DBshow "+filename+' '+dbshow_reads
stream = subprocess.Popen(DBshow_cmd.split(),
                                  stdout=subprocess.PIPE,bufsize=1)
reads_queried = parse_read(stream.stdout)
read_dict = {}
for read_id,read in itertools.izip(reads,reads_queried):
    rdlen = len(read[1])
#     print read
    read_dict[read_id] = read

complement = {'A':'T','C': 'G','T':'A', 'G':'C','a':'t','t':'a','c':'g','g':'c'}

def reverse_complement(string):
    return "".join(map(lambda x:complement[x],reversed(string)))

def get_string(path):
    #print path
    ret_str = ''
    for itm in path:
#         print itm
        read_id,rd_orientation = itm[0].split("_")
        if rd_orientation == '1':
            assert itm[1][0] >= itm[1][1]
#             print itm
            str_st = itm[1][1]
            str_end = itm[1][0]
            read_str = read_dict[int(read_id.lstrip("B"))][1][str_st:str_end]
        else:
            assert itm[1][0] <= itm[1][1]
            str_st = itm[1][0]
            str_end = itm[1][1]
            read_str = reverse_complement(read_dict[int(read_id.lstrip("B"))][1][str_st:str_end])
#         print str_st,str_end
#         print read_id
#         print read_dict[int(read_id)][str_st:str_end]
#         print read_str
        print 'read len',len(read_str)
        ret_str += read_str
    print len(path), len(ret_str)
    return ret_str




vertices_of_interest = set([x for x in in_graph if in_graph.in_degree(x) != 1 or in_graph.out_degree(x) != 1])

read_tuples = {}

for vert in vertices_of_interest:

    vert_id, vert_or = vert.split("_")
    if vert_or == '1':
        continue
    vert_len = len(read_dict[int(vert_id)][1])
#     print vert_len
    read_starts = [(in_graph.edge[x][vert]['read_b_start']) for x in in_graph.predecessors(vert)]
    read_ends = [(in_graph.edge[vert][x]['read_a_start']) for x in in_graph.successors(vert)]
    if read_starts:
        read_start = max(read_starts)
    else:
        read_start = 0
    if read_ends:
        read_end = min(read_ends)
    else:
        read_end = vert_len
    read_tuples[vert] = (read_start,read_end)
    print read_starts, read_ends, vert


for vert in vertices_of_interest:

    vert_id, vert_or = vert.split("_")
    if vert_or == '1':
        read_tuples[vert] = read_tuples[vert_id+"_0"]


start_vertices = [x for x in vertices_of_interest if in_graph.in_degree(x) == 0 or in_graph.out_degree(x) > 1]
h = nx.DiGraph()

read_tuples_raw = {}
for vertex in vertices_of_interest:
    successors = in_graph.successors(vertex)
    if successors:
        succ = successors[0]
        d =  in_graph.get_edge_data(vertex,succ)
        read_tuples_raw[vertex] = (d['read_a_start_raw'], d['read_a_end_raw'])
    else:
        predecessors = in_graph.predecessors(vertex)
        if not len(predecessors) == 0:
            pred = predecessors[0]
            d =  in_graph.get_edge_data(pred,vertex)
            read_tuples_raw[vertex] = (d['read_b_start_raw'], d['read_b_end_raw'])
        else:
            read_tuples_raw[vertex] = (0,0)



for vertex in vertices_of_interest:
    h.add_node(vertex)
    if vertex.split("_")[1] == '0':
        path_var = [(vertex,(read_tuples[vertex][0], read_tuples[vertex][1]))]
    else:
        path_var = [(vertex,(read_tuples[vertex][1], read_tuples[vertex][0]))]
    #print path_var
    segment = get_string(path_var)
    h.node[vertex]['start_read'] = path_var[0][1][0]
    h.node[vertex]['end_read'] = path_var[0][1][1]
    h.node[vertex]['path'] = [vertex]
    h.node[vertex]['segment'] = segment

vertices_used = set([x for x in h.nodes()])
contig_no = 1
for start_vertex in vertices_of_interest:
    first_out_vertices = in_graph.successors(start_vertex)
    print start_vertex, first_out_vertices
    for vertex in first_out_vertices:
        predecessor = start_vertex
        start_vertex_id,start_vertex_or = start_vertex.split("_")
        cur_vertex = vertex
        if start_vertex_or == '0':
            cur_path = [(start_vertex,(read_tuples[start_vertex][1],
                                       in_graph.edge[start_vertex][cur_vertex]['read_a_start']))]
        elif start_vertex_or == '1':
            cur_path = [(start_vertex,(read_tuples[start_vertex][0],
                                       in_graph.edge[start_vertex][cur_vertex]['read_a_start']))]

        while cur_vertex not in vertices_of_interest:
            successor = in_graph.successors(cur_vertex)[0]
            start_point = in_graph.edge[predecessor][cur_vertex]['read_b_start']
            end_point = in_graph.edge[cur_vertex][successor]['read_a_start']
            cur_path.append((cur_vertex,(start_point,end_point)))
            vertices_used.add(cur_vertex)
            predecessor = cur_vertex
            cur_vertex = successor

        stop_vertex_id, stop_vertex_or = cur_vertex.split("_")
        if stop_vertex_or == '0':
            cur_path.append((cur_vertex,(in_graph.edge[predecessor][cur_vertex]['read_b_start'],
                        read_tuples[cur_vertex][0])))
        elif stop_vertex_or == '1':
            cur_path.append((cur_vertex,(in_graph.edge[predecessor][cur_vertex]['read_b_start'],
                        read_tuples[cur_vertex][1])))


        node_name = str(contig_no)
        h.add_node(node_name)
        contig_no += 1
#         print cur_path
        node_path = [x[0] for x in cur_path]
        h.node[node_name]['path'] = node_path
        h.node[node_name]['start_read'] = path_var[0][1][0]
        h.node[node_name]['end_read'] = path_var[-1][1][1]
        h.node[node_name]['segment'] = get_string(cur_path)
        h.add_edges_from([(start_vertex,node_name),(node_name,cur_vertex)])
#         paths.append(cur_path)

#print read_tuples

while set(in_graph.nodes())-vertices_used:
    vert = list(set(in_graph.nodes())-vertices_used)[0]
    vert_id,vert_or = vert.split("_")
    if vert_or == '0':
        read_start = min( min([(in_graph.edge[x][vert]['read_b_start']) for x in in_graph.predecessors(vert)]),
                         max([(in_graph.edge[vert][x]['read_a_start']) for x in in_graph.successors(vert)]))
        read_end = max( min([(in_graph.edge[x][vert]['read_b_start']) for x in in_graph.predecessors(vert)]),
                         max([(in_graph.edge[vert][x]['read_a_start']) for x in in_graph.successors(vert)]))
        vertRC = vert_id+"_1"
    else:
        read_start = max( min([(in_graph.edge[x][vert]['read_b_start']) for x in in_graph.predecessors(vert)]),
                         max([(in_graph.edge[vert][x]['read_a_start']) for x in in_graph.successors(vert)]))
        read_end = min( min([(in_graph.edge[x][vert]['read_b_start']) for x in in_graph.predecessors(vert)]),
                         max([(in_graph.edge[vert][x]['read_a_start']) for x in in_graph.successors(vert)]))
        vertRC = vert_id+"_0"

    successor_start = in_graph.successors(vert)[0]
    d =  in_graph.get_edge_data(vert,successor_start)
    read_tuples_raw[vert] = (d['read_a_start_raw'], d['read_a_end_raw'])

    successor_start = in_graph.successors(vertRC)[0]
    d =  in_graph.get_edge_data(vertRC,successor_start)
    read_tuples_raw[vertRC] = (d['read_a_start_raw'], d['read_a_end_raw'])

    h.add_node(vert)
    node_path = [vert]
    h.node[vert]['path'] = node_path
    h.node[vert]['start_read'] = read_start
    h.node[vert]['end_read'] = read_end
    h.node[vert]['segment'] = get_string([(vert,(read_start, read_end))])
    vertices_used.add(vert)

    first_out_vertices = in_graph.successors(vert)
    for vertex in first_out_vertices:
        predecessor = vert
        cur_vertex = vertex
        cur_path = []
        while cur_vertex != vert:
            successor = in_graph.successors(cur_vertex)[0]
            start_point = in_graph.edge[predecessor][cur_vertex]['read_b_start']
            end_point = in_graph.edge[cur_vertex][successor]['read_a_start']
            cur_path.append((cur_vertex,(start_point,end_point)))
            vertices_used.add(cur_vertex)
            predecessor = cur_vertex
            cur_vertex = successor
        node_name = str(contig_no)
        h.add_node(node_name)
        contig_no += 1
#         print cur_path

        node_path = [x[0] for x in cur_path]
        h.node[node_name]['path'] = node_path
        h.node[node_name]['start_read'] = path_var[0][1][0]
        h.node[node_name]['end_read'] = path_var[-1][1][1]
        h.node[node_name]['segment'] = get_string(cur_path)
        h.add_edges_from([(vert,node_name),(node_name,vert)])

    if vertRC not in vertices_used:
        h.add_node(vertRC)
        h.node[vertRC]['segment'] = get_string([(vertRC,(read_end, read_start))])
        h.node[vertRC]['path'] = [vertRC]
        h.node[vertRC]['start_read'] = read_end
        h.node[vertRC]['end_read'] = read_start

        vertices_used.add(vertRC)
        first_out_vertices = in_graph.successors(vertRC)
        for vertex in first_out_vertices:
            predecessor = vertRC
            cur_vertex = vertex
            cur_path = []
            while cur_vertex != vertRC:
                successor = in_graph.successors(cur_vertex)[0]
                start_point = in_graph.edge[predecessor][cur_vertex]['read_b_start']
                end_point = in_graph.edge[cur_vertex][successor]['read_a_start']
                cur_path.append((cur_vertex,(start_point,end_point)))
                vertices_used.add(cur_vertex)
                predecessor = cur_vertex
                cur_vertex = successor
            node_name = str(contig_no)
            h.add_node(node_name)
            contig_no += 1
    #         print cur_path

            node_path = [x[0] for x in cur_path]
            h.node[node_name]['path'] = node_path
            h.node[node_name]['start_read'] = path_var[0][1][0]
            h.node[node_name]['end_read'] = path_var[-1][1][1]
            h.node[node_name]['segment'] = get_string(cur_path)
            print len(cur_path)
            h.add_edges_from([(vertRC,node_name),(node_name,vertRC)])



outfile = '/data/pacbio_assembly/pb_data/NCTC/'+NCTCname+'/'+NCTCname + ".edges.list"

vert_to_merge = [x for x in h.nodes() if len(h.successors(x)) == 1 and len(h.predecessors(h.successors(x)[0])) == 1 and
 len(nx.node_connected_component(h.to_undirected(), x)) > 2]

while True:

    vert_to_merge = [x for x in h.nodes() if len(h.successors(x)) == 1 and len(h.predecessors(h.successors(x)[0])) == 1 and
 len(nx.node_connected_component(h.to_undirected(), x)) > 2]

    if not vert_to_merge:
        break
    vert = vert_to_merge[0]
    #print vert,
    succ = h.successors(vert)[0]
    preds = h.predecessors(vert)
    h.node[succ]['segment'] =  h.node[vert]['segment'] + h.node[succ]['segment']
    h.node[succ]['path'] = h.node[vert]['path'] + h.node[succ]['path'][1:]
    for pred in preds:
        #print pred, succ
        h.add_edges_from([(pred,succ)])
        h.remove_edge(pred,vert)
    h.remove_edge(vert,succ)
    h.remove_node(vert)

for  i, vert in enumerate(h.nodes()):
    print i,len(h.node[vert]['path'])

with open(outfile, 'w') as f:
    for i,node in enumerate(h.nodes()):
        #print node
        #print h.node[node]
        path = h.node[node]['path']

        f.write('>Unitig%d\n'%(i))
        if len(path) == 1:
            #print path[0]
            f.write(' '.join([path[0].split('_')[0], path[0].split('_')[1], str(read_tuples_raw[path[0]][0]), str(read_tuples_raw[path[0]][1])]) + '\n')
        for j in range(len(path)-1):
            nodeA = path[j].lstrip("B")
            nodeB = path[j+1].lstrip("B")

            d =  in_graph.get_edge_data(path[j],path[j+1])

            f.write('%s %s %s %s %d %d %d %d %d\n'%(nodeA.split('_')[0],nodeA.split('_')[1]  , nodeB.split('_')[0],
                    nodeB.split('_')[1], -d['read_a_start_raw'] + d['read_a_end_raw'] - d['read_b_start_raw'] + d['read_b_end_raw'],
                    d['read_a_start_raw'], d['read_a_end_raw'], d['read_b_start_raw'], d['read_b_end_raw']))

out_graphml_name = '/data/pacbio_assembly/pb_data/NCTC/'+NCTCname+'/'+NCTCname+'_draft.graphml'



gfaname = '/data/pacbio_assembly/pb_data/NCTC/'+NCTCname+'/'+NCTCname+'_draft_python.gfa'
consensus_name = sys.argv[3]

consensus_contigs = []
try:
    with open(consensus_name) as f:
        for line in f:
            if line[0] != '>':
                consensus_contigs.append(line.strip())
except:
    pass
for  i, vert in enumerate(h.nodes()):
    print i,len(h.node[vert]['path']), len(h.node[vert]['segment']), len(consensus_contigs[i])


with open(gfaname,'w') as f:
    f.write("H\tVN:Z:1.0\n")
    for i,vert in enumerate(h.nodes()):
        if len(consensus_contigs) > 0:
            seg = consensus_contigs[i]
        else:
            seg = h.node[vert]['segment']

        seg_line = "S\t"+vert+"\t"+seg + '\n'
        f.write(seg_line)
    for edge in h.edges():
        edge_line = "L\t"+edge[0]+"\t+\t"+edge[1]+"\t+\t0M\n"
        f.write(edge_line)

#last =  h.nodes()[-1]
#print h.node[last]
#path_last = h.node[last]['path']



#for i in range(len(path_last)-1):
#    read_a = path_last[i]
#    read_b = path_last[i+1]
#    print read_a, read_b, in_graph.edge[read_a][read_b]

for i,node in enumerate(h.nodes()):
     h.node[node]['path'] = ';'.join(h.node[node]['path'])
nx.write_graphml(h,out_graphml_name)






#!/usr/bin/env python

import networkx as nx
import sys
from collections import Counter

def linearize(filename):
    graph_name = filename.split('.')[0]+'.graphml'
    g = nx.read_graphml(graph_name)
    
    print nx.info(g)
    
    # get first strong connected component
    
    con = list(nx.strongly_connected_component_subgraphs(g))
    
    con.sort(key = lambda x:len(x), reverse = True)
    print [len(item) for item in con]
    
    print nx.info(con[0])
    
    dfs_edges = list(nx.dfs_edges(con[0]))

    
    dfs_edges.append((dfs_edges[-1][-1], dfs_edges[0][0]))
    
    #print dfs_edges
    
    with open(filename.split('.')[0]+'.linear.edges', 'w') as f:
        for item in dfs_edges:
            f.write(item[0] + ' ' + item[1] + ' ' + str(con[0].edge[item[0]][item[1]]['ew'])+'\n')
            


filename = sys.argv[1]
linearize(filename)



#!/usr/bin/env python

import sys
min_len_aln = 1000


with sys.stdin as f:
    for l in f:
        l = l.strip().split()
        if len(l) != 2:
            continue

        read_id = l[0]
        seq = l[1]
        
        print read_id,seq
        
        #if len(seq) > max_len:
        #    seq = seq[:max_len-1]
        
        if read_id not in ("+", "-", "*"):
            if len(seq) >= min_len_aln:
                if len(seqs) == 0:
                    seqs.append(seq) #the "seed"
                    seed_id = l[0]
                if read_id not in read_ids: #avoidng using the same read twice. seed is used again here by design
                    seqs.append(seq)
                    read_ids.add(read_id)
        elif l[0] == "+":
            if len(seqs) >= min_cov_aln:
                seqs = seqs[:1] + sorted(seqs[1:], key=lambda x: -len(x))
                yield (seqs[:max_n_read], seed_id, config) 
            #seqs_data.append( (seqs, seed_id) ) 
            seqs = []
            read_ids = set()
            seed_id = None
        elif l[0] == "*":
            seqs = []
            read_ids = set()
            seed_id = None
        elif l[0] == "-":
            #yield (seqs, seed_id)
            #seqs_data.append( (seqs, seed_id) )
            break



#!/usr/bin/env python
import networkx as nx
import sys
import itertools

filename = sys.argv[1]
outfile = filename.split('.')[0] + ".edges.list"

g = nx.read_graphml(filename)
print nx.info(g)


def get_circle(g,node,vertices_of_interest):
    cur_path = [node]
    cur_vertex = g.successors(node)[0]

    i = 0
    while cur_vertex != node:
        cur_path.append(cur_vertex)
        try:
            assert len(g.successors(cur_vertex)) == 1
        except:
            print g.successors(cur_vertex), cur_vertex, node
            print cur_vertex in vertices_of_interest
            raise
        successor = g.successors(cur_vertex)[0]
        cur_vertex = successor
        
    cur_path.append(cur_vertex)
    


    return cur_path

    
def get_unitigs(g):
    paths = []
    num_paths = 0
    node_set = set(g.nodes())

    
    vertices_of_interest = set([x for x in g if g.in_degree(x) != 1 or g.out_degree(x) != 1])
    vertices_used = set(vertices_of_interest)
    for start_vertex in vertices_of_interest:
        first_out_vertices = g.successors(start_vertex)
        print first_out_vertices
        for vertex in first_out_vertices:
            cur_path = [start_vertex]
            cur_vertex = vertex
            while cur_vertex not in vertices_of_interest:
                successor = g.successors(cur_vertex)[0]
                cur_path.append(cur_vertex)
                predecessor = cur_vertex
                cur_vertex = successor
                
            cur_path.append(cur_vertex)
            vertices_used = vertices_used.union(set(cur_path))
            paths.append(cur_path)

    print len(node_set)
    print len(vertices_used)

    while len(node_set-vertices_used) > 0:
        node = list(node_set-vertices_used)[0]
        # print list(node_set-vertices_used)
        # # print vertices_of_interest
        # # print len(node_set-vertices_used)
        # break
        path = get_circle(g, node, vertices_of_interest)
        vertices_used = vertices_used.union(set(path))
        if len(path) > 1:
            paths.append(path)
    print len(paths) 
    # print paths
    print "paths"
    return paths
        
        
paths = get_unitigs(g)

print len(paths)

h = nx.DiGraph()
for i, path in enumerate(paths):
    h.add_node(i)
    h.node[i]['path'] = path

vertices_of_interest = set([x for x in g if g.in_degree(x) != 1 or g.out_degree(x) != 1])
for vertex in vertices_of_interest:
    successors = [x for x in h.nodes() if h.node[x]['path'][0] == vertex]
    predecessors = [x for x in h.nodes() if h.node[x]['path'][-1] == vertex]
    print successors,predecessors
    assert len(successors)==1 or len(predecessors)==1
    for succ, pred in itertools.product(successors,predecessors):
        h.add_edge(pred,succ)

    # if vertex.split('_') == '0':
    #     if len(predecessors) == 1:
    #         for succ in successors:
    #             rel_suc = h.node[succ]['path'][1]
    #             d = g.get_edge_data(vertex,rel_suc)
    #             h.edge[predecessors[0]][succ]['start_pos'] = d['read_a_end']
    #             h.edge[predecessors[0]][succ]['weight'] = d['read_a_end_raw'] - d['read_a_start']
    #     if len(successors) == 1:
    #         for pred in predecessors:
    #             rel_pred = h.node[pred]['path'][-2]
    #             d = g.get_edge_data(rel_pred,vertex)
    #             h.edge[predecessors[0]][succ][''] = d['read_a_end_raw'] - d['read_a_start']


with open(outfile, 'w') as f:
    for i,path in enumerate(paths):
        f.write('>Unitig%d\n'%(i))
        for j in range(len(path)-1):
            nodeA = path[j].lstrip("B")
            nodeB = path[j+1].lstrip("B")

            d =  g.get_edge_data(path[j],path[j+1])

            f.write('%s %s %s %s %d %d %d %d %d\n'%(nodeA.split('_')[0],nodeA.split('_')[1]  , nodeB.split('_')[0], 
                    nodeB.split('_')[1], -d['read_a_start_raw'] + d['read_a_end_raw'] - d['read_b_start_raw'] + d['read_b_end_raw'], 
                    d['read_a_start_raw'], d['read_a_end_raw'], d['read_b_start_raw'], d['read_b_end_raw']))



    f.close()







#!/usr/bin/env python
"""
Convert FASTA to FASTQ file with a static

Usage:
$ ./fasta_to_fastq NAME.fasta NAME.fastq
"""

import sys, os
from Bio import SeqIO

# Get inputs
fa_path = sys.argv[1]
fq_path = sys.argv[2]

# make fastq
with open(fa_path, "r") as fasta, open(fq_path, "w") as fastq:
    for record in SeqIO.parse(fasta, "fasta"):
        record.letter_annotations["phred_quality"] = [40] * len(record)
        SeqIO.write(record, fastq, "fastq")



import json
import os
import sys
import subprocess

base_dir = '/data/pacbio_assembly/pb_data/NCTC/'
bact_dict = json.load(open(base_dir+'NCTC.json'))

#bacterium_of_interest='NCTC7972'

bacterium_of_interest=sys.argv[1]

if len(sys.argv) > 2:
	bact_dict=sys.argv[2]

bact_name="_".join(bact_dict[bacterium_of_interest]['Species'][0].split())

cmd_base = 'ascp -QT -l 1000m -i /data/pacbio_assembly/pb_data/asperaweb_id_dsa.openssh era-fasp@fasp.ega.ebi.ac.uk:vol1/'
dest_dir = base_dir+bacterium_of_interest+'/'

os.system('mkdir -p '+dest_dir)

for run, file_list in bact_dict[bacterium_of_interest]['file_paths'].items():
    for file_path in  file_list:
        cmd = cmd_base+file_path+' '+dest_dir
        print cmd
        os.system(cmd)

dest_fasta_name = dest_dir+bact_name

dextract_cmd = 'dextract -o'+dest_fasta_name

bax_files = [x for x in os.listdir(dest_dir) if x.endswith('.bax.h5')]

for bax_file in bax_files:
	dextract_cmd +=  " " + dest_dir+bax_file

print dextract_cmd

try:
    subprocess.check_output(dextract_cmd.split())
    print 'dextract done. deleting .bax.h5 files'
    os.system('rm '+dest_dir+'*.bax.h5')
    print 'removing .quiva files'
    os.system('rm '+dest_dir+'*.quiva')
except:
    print 'error'






#!/usr/bin/env python

import networkx as nx
import random
import sys
from collections import Counter
import ujson



def dead_end_clipping(G,threshold):
#     H=nx.DiGraph()
    H = G.copy()
    start_nodes = set([x for x in H.nodes() if H.in_degree(x) ==0])
    
    for st_node in start_nodes:
        cur_path = [st_node]
        
        if len(H.successors(st_node)) == 1:
            cur_node = H.successors(st_node)[0]
            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1 and len(cur_path) < threshold + 2:
                cur_path.append(cur_node)
                cur_node = H.successors(cur_node)[0]
                
            
        if len(cur_path) <= threshold:
            for vertex in cur_path:
                H.remove_node(vertex)
    
    end_nodes = set([x for x in H.nodes() if H.out_degree(x) ==0])
    
    for end_node in end_nodes:
        cur_path = [end_node]
        if len(H.predecessors(end_node)) == 1:
            cur_node = H.predecessors(end_node)[0]
            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1 and len(cur_path) < threshold + 2:
                cur_path.append(cur_node)
                cur_node = H.predecessors(cur_node)[0]
            
        if len(cur_path) <= threshold:
            for vertex in cur_path:
                H.remove_node(vertex)

    return H



# In[9]:

def z_clipping(G,threshold,in_hinges,out_hinges,print_z = False):
    H = G.copy()
    
    start_nodes = set([x for x in H.nodes() if H.out_degree(x) > 1 and x not in out_hinges])
    
    for st_node in start_nodes:
        for sec_node in H.successors(st_node):
            
            if H.out_degree(st_node) == 1:
                break
            
            cur_node = sec_node
            cur_path = [[st_node,cur_node]]

            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:
                cur_path.append([cur_node,H.successors(cur_node)[0]])
                cur_node = H.successors(cur_node)[0]
                
                if len(cur_path) > threshold + 1:
                    break
            
            if len(cur_path) <= threshold and H.in_degree(cur_node) > 1 and H.out_degree(st_node) > 1 and cur_node not in in_hinges:
                if print_z:
                    print cur_path
                
                for edge in cur_path:
                    H.remove_edge(edge[0],edge[1])
                for j in range(len(cur_path)-1):
                    H.remove_node(cur_path[j][1])
    
    end_nodes = set([x for x in H.nodes() if H.in_degree(x) > 1 and x not in in_hinges])
    
    for end_node in end_nodes:
        for sec_node in H.predecessors(end_node):
            
            if H.in_degree(end_node) == 1:
                break

            
            cur_node = sec_node
            cur_path = [[cur_node,end_node]]
        
            while H.in_degree(cur_node) == 1 and H.out_degree(cur_node) == 1:
                cur_path.append([H.predecessors(cur_node)[0],cur_node])
                cur_node = H.predecessors(cur_node)[0]
                
                if len(cur_path) > threshold + 1:
                    break
            
            if len(cur_path) <= threshold and H.out_degree(cur_node) > 1 and H.in_degree(end_node) > 1 and cur_node not in out_hinges:
                if print_z:
                    print cur_path
                for edge in cur_path:
                    H.remove_edge(edge[0],edge[1])
                for j in range(len(cur_path)-1):
                    H.remove_node(cur_path[j][0])

    return H





def merge_path(g,in_node,node,out_node):
        
    g.add_edge(in_node,out_node,hinge_edge = -1,false_positive = 0)        
    g.remove_node(node)



def merge_a_to_b(g,node_a,node_b):
        
    if node_a not in g.nodes() or node_b not in g.nodes():
        return

    for node in g.predecessors(node_a):
        if node != node_b:
            g.add_edge(node,node_b,hinge_edge = 1,false_positive = 0)

    for node in g.successors(node_a):
        if node != node_b:
            g.add_edge(node_b,node,hinge_edge = 1,false_positive = 0)
   
    g.remove_node(node_a)


def random_condensation(G,n_nodes):

    g = G.copy()
    
    max_iter = 20000
    iter_cnt = 0
    
    while len(g.nodes()) > n_nodes and iter_cnt < max_iter:
        
        iter_cnt += 1

        node = g.nodes()[random.randrange(len(g.nodes()))]

        if g.in_degree(node) == 1 and g.out_degree(node) == 1:

            in_node = g.in_edges(node)[0][0]
            out_node = g.out_edges(node)[0][1]
            if g.out_degree(in_node) == 1 and g.in_degree(out_node) == 1:
                if in_node != node and out_node != node and in_node != out_node:
                    #print in_node, node, out_node
#                     merge_path(g,in_node,node,out_node)
                    
                    bad_node=False
                    for in_edge in g.in_edges(node):
                        if g.edge[in_edge[0]][in_edge[1]]['false_positive']==1:
                            bad_node=True
                    for out_edge in g.out_edges(node):
                        if g.edge[out_edge[0]][out_edge[1]]['false_positive']==1:
                            bad_node=True 
                    if not bad_node:
                        #print in_node, node, out_node
                        merge_path(g,in_node,node,out_node)
    
    if iter_cnt >= max_iter:
        print "couldn't finish sparsification"+str(len(g.nodes()))
                        
    return g



def add_groundtruth(g,json_file,in_hinges,out_hinges):
    
    mapping = ujson.load(json_file)

    print 'getting mapping'
    mapped_nodes=0
    print str(len(mapping)) 
    print str(len(g.nodes()))
    
    slack = 500
           
            
    for node in g.nodes():
        # print node
        node_base=node.split("_")[0]
        # print node_base

        #print node
        if mapping.has_key(node_base):
            g.node[node]['aln_start'] = min (mapping[node_base][0][0],mapping[node_base][0][1])
            g.node[node]['aln_end'] = max(mapping[node_base][0][1],mapping[node_base][0][0])
#             g.node[node]['chr'] = mapping[node_base][0][2]
            mapped_nodes+=1
        else:
            # pass
            g.node[node]['aln_start'] = 0
            g.node[node]['aln_end'] = 0
#             g.node[node]['aln_strand'] = 0
            
        if node in in_hinges or node in out_hinges:
            g.node[node]['hinge'] = 1
        else:
            g.node[node]['hinge'] = 0


    for edge in g.edges_iter():
        in_node=edge[0]
        out_node=edge[1]

#         if ((g.node[in_node]['aln_start'] < g.node[out_node]['aln_start'] and  
#             g.node[out_node]['aln_start'] < g.node[in_node]['aln_end']) or 
#             (g.node[in_node]['aln_start'] < g.node[out_node]['aln_end'] and
#             g.node[out_node]['aln_end'] < g.node[in_node]['aln_end'])):
#             g.edge[in_node][out_node]['false_positive']=0
#         else:
#             g.edge[in_node][out_node]['false_positive']=1

        
        
        if ((g.node[in_node]['aln_start'] < g.node[out_node]['aln_start'] and  
            g.node[out_node]['aln_start'] < g.node[in_node]['aln_end']) or 
            (g.node[in_node]['aln_start'] < g.node[out_node]['aln_end'] and
            g.node[out_node]['aln_end'] < g.node[in_node]['aln_end'])):
            g.edge[in_node][out_node]['false_positive']=0
        else:
            g.edge[in_node][out_node]['false_positive']=1
            
    return g
   





def read_graph(edges_file,hg_file,gt_file, hinge_file):


    prefix = edges_file.split('.')[0]

    with open(gt_file) as f:
        read_dict = ujson.load(f)

    g = nx.DiGraph()

    hinge_nodes = []
    hinge_pos = {}


    with open (hinge_file) as f:
        for lines in f:
            lines1=lines.split()
            hinge_nodes.append(lines1[0] + "_0_" + lines1[1])
            hinge_nodes.append(lines1[0] + "_1_" + lines1[1])
            

            # if lines1[0] not in hinge_pos:
            #     hinge_pos[lines]
            # hinge_pos[lines1[0]] = lines1[1]


    # with open (hg_file) as f:
    #     for lines in f:
    #         lines1=lines.split()
    #         g.add_node(lines1[0] + "_" + lines1[2])
    #         g.add_node(lines1[1] + "_" + lines1[3])
    #         if lines1[0] in read_dict:
    #             g.node[lines1[0] + "_" + lines1[2]]['aln_start']=min(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
    #             g.node[lines1[0] + "_" + lines1[2]]['aln_end']=max(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
    #         else:
    #             g.node[lines1[0] + "_" + lines1[2]]['aln_start']=0
    #             g.node[lines1[0] + "_" + lines1[2]]['aln_end']=0
    #         if lines1[1] in read_dict:
    #             g.node[lines1[1] + "_" + lines1[3]]['aln_start']=min(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
    #             g.node[lines1[1] + "_" + lines1[3]]['aln_end']=max(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
    #         else:
    #             g.node[lines1[1] + "_" + lines1[3]]['aln_start']=0
    #             g.node[lines1[1] + "_" + lines1[3]]['aln_end']=0

    #         if lines1[0] in hinge_nodes:
    #             g.node[lines1[0] + "_" + lines1[2]]['active']=2
    #         else:
    #             g.node[lines1[0] + "_" + lines1[2]]['active']=1

    #         if lines1[1] in hinge_nodes:
    #             g.node[lines1[1] + "_" + lines1[3]]['active']=2
    #         else:
    #             g.node[lines1[1] + "_" + lines1[3]]['active']=int(lines1[4])


    #         g.add_edge(lines1[0] + "_" + lines1[2], lines1[1] + "_" + lines1[3], rev = int(lines1[5]))


    # need to construct double stranded hinge graph, so that proper mapping can be found

    with open (hg_file) as f:
        for lines in f:
            lines1=lines.split()

            nodeA0 = lines1[0] + "_0_"+ lines1[2]
            nodeA1 = lines1[0] + "_1_"+ lines1[2]
            nodeB0 = lines1[1] + "_0_"+ lines1[3]
            nodeB1 = lines1[1] + "_1_"+ lines1[3]            

            nodeA0short = lines1[0] + "_0"
            nodeA1short = lines1[0] + "_1"
            nodeB0short = lines1[1] + "_0"
            nodeB1short = lines1[1] + "_1"

            g.add_node(nodeA0)
            g.add_node(nodeA1)
            g.add_node(nodeB0)
            g.add_node(nodeB1)

            if nodeA0short not in hinge_pos:
                hinge_pos[nodeA0short] = [int(lines1[2])]
                hinge_pos[nodeA1short] = [int(lines1[2])]
            elif lines1[2] not in hinge_pos[nodeA0short]:
                hinge_pos[nodeA0short].append(int(lines1[2]))
                hinge_pos[nodeA1short].append(int(lines1[2]))

            if nodeB0 not in hinge_pos:
                hinge_pos[nodeB0short] = [int(lines1[3])]
                hinge_pos[nodeB1short] = [int(lines1[3])]
            elif lines1[3] not in hinge_pos[nodeB0short]:
                hinge_pos[nodeB0short].append(int(lines1[3]))
                hinge_pos[nodeB1short].append(int(lines1[3]))


            if lines1[0] in read_dict:
                g.node[lines1[0] + "_0_"+ lines1[2]]['aln_start']=min(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
                g.node[lines1[0] + "_0_"+ lines1[2]]['aln_end']=max(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
                g.node[lines1[0] + "_1_"+ lines1[2]]['aln_start']=min(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
                g.node[lines1[0] + "_1_"+ lines1[2]]['aln_end']=max(read_dict[lines1[0]][0][0],read_dict[lines1[0]][0][1])
            else:
                g.node[lines1[0] + "_0_"+ lines1[2]]['aln_start']=0
                g.node[lines1[0] + "_0_"+ lines1[2]]['aln_end']=0
                g.node[lines1[0] + "_1_"+ lines1[2]]['aln_start']=0
                g.node[lines1[0] + "_1_"+ lines1[2]]['aln_end']=0
            if lines1[1] in read_dict:
                g.node[lines1[1] + "_0_"+ lines1[3]]['aln_start']=min(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
                g.node[lines1[1] + "_0_"+ lines1[3]]['aln_end']=max(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
                g.node[lines1[1] + "_1_"+ lines1[3]]['aln_start']=min(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
                g.node[lines1[1] + "_1_"+ lines1[3]]['aln_end']=max(read_dict[lines1[1]][0][0],read_dict[lines1[1]][0][1])
            else:
                g.node[lines1[1] + "_0_"+ lines1[3]]['aln_start']=0
                g.node[lines1[1] + "_0_"+ lines1[3]]['aln_end']=0
                g.node[lines1[1] + "_1_"+ lines1[3]]['aln_start']=0
                g.node[lines1[1] + "_1_"+ lines1[3]]['aln_end']=0

            if nodeA0 in hinge_nodes:
                g.node[nodeA0]['active']=2
                g.node[nodeA1]['active']=2
            else:
                g.node[nodeA0]['active']=1
                g.node[nodeA1]['active']=1

            if nodeB0 in hinge_nodes:
                g.node[nodeB0]['active']=2
                g.node[nodeB1]['active']=2
            else:
                g.node[nodeB0]['active']=int(lines1[4])
                g.node[nodeB1]['active']=int(lines1[4])


            if int(lines1[5]) == 1: # reverse match
                g.add_edge(nodeA0, nodeB1)
                g.add_edge(nodeA1, nodeB0)
            else:
                g.add_edge(nodeA0,nodeB0)
                g.add_edge(nodeA1,nodeB1)

    
    
    # nx.write_graphml(g, filename.split('.')[0]+'_hgraph.graphml')
    
    # for c in nx.connected_components(g):
        # print len(c)

    hinge_mapping = {}

    for c in nx.weakly_connected_components(g):

        if len(c) > 10:

            component_sink = -1

            for node in c:
                if g.out_degree(node) == 0 and g.node[node]['active']== 2 and component_sink == -1:
                    component_sink = node
                elif g.out_degree(node) == 0 and g.node[node]['active']== 2 and g.in_degree(node) > g.in_degree(component_sink):
                    component_sink = node

            if component_sink != -1:
                g.node[component_sink]['active'] = 3
            else:
                component_sink = list(c)[0]

                # sink_shortname = component_sink.split('_')[0]

            for node in c:
                hinge_mapping[node] = component_sink

        else:

            for node in c:
                g.node[node]['active'] = -1


    nx.write_graphml(g, hg_file.split('.')[0]+'_hgraph2.graphml')


    # print nx.number_weakly_connected_components(g)
    # print nx.number_strongly_connected_components(g)


    G = nx.DiGraph()

    merging = 1

    if merging == 0:

        with open (edges_file) as f:
            for lines in f:
                lines1=lines.split()

                if len(lines1) < 6:
                    continue

                if int(lines1[5]) != 0:

                    if int(lines1[5]) == 1: 
                        # nodeB_id = lines1[1]+"_"+lines1[4]
                        # hingepos = int(lines1[6])
                        nodeA_id = lines1[0] + "_" + lines1[3]

                        hinge_node = lines1[1]+"_"+lines1[4] + '_' + lines1[6]

                        print hinge_node

                        eff_hinge = hinge_mapping[hinge_node]

                        eff_b = eff_hinge.split('_')

                        if eff_b[0] + "_" + eff_b[1] != lines1[0] + "_" + lines1[3]:
                            G.add_edge(lines1[0] + "_" + lines1[3], eff_b[0] + "_" + eff_b[1],hinge_edge=1)
                            G.add_edge(eff_b[0] + "_" + str(1-int(eff_b[1])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=1)
                        else:

                            G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],hinge_edge=1)
                            G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=1)


                    elif int(lines1[5]) == -1:

                        hinge_node = lines1[0]+"_"+lines1[3] + '_' + lines1[6]

                        eff_hinge = hinge_mapping[hinge_node]

                        eff_b = eff_hinge.split('_')

                        if eff_b[0] + "_" + eff_b[1] != lines1[1] + "_" + lines1[4] :
                            G.add_edge(eff_b[0] + "_" + eff_b[1], lines1[1] + "_" + lines1[4] ,hinge_edge=1)
                            G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), eff_b[0] + "_" + str(1-int(eff_b[1])),hinge_edge=1)
                        else:
                            G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],hinge_edge=1)
                            G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=1)


                    # if nodeA_id in hinge_pos:
                    #     print "found A"
                    # else:
                    #     print "didnt find A"


                    # if nodeB_id in hinge_pos:

                    #     print "Node B of hinged match IS in hinge_pos"

                    #     hinge_found = False

                    #     hingepos = hinge_pos[nodeB_id][0]
                    #     for candidate_pos in hinge_pos[nodeB_id]:
                    #         if (abs(candidate_pos - int(lines1[8][1:])) < 200):
                    #             hingepos = candidate_pos

                    #             print "Matching hinge found"

                    #             hinge_found = True

                    #     if not hinge_found:
                    #         print "not found"
                    #         print lines1[8][1:], hinge_pos[nodeB_id]


                    #     hinge_node = nodeB_id + '_' + str(hingepos)
                    #     eff_hinge = hinge_mapping[hinge_node]

                    #     eff_b = eff_hinge.split('_')

                    #     G.add_edge(lines1[0] + "_" + lines1[3], eff_b[0] + "_" + eff_b[1],hinge_edge=1)
                    #     G.add_edge(eff_b[0] + "_" + str(1-int(eff_b[1])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=1)


                else:

                    G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],hinge_edge=int(lines1[5]))
                    G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))



    else:

        to_be_merged = []

        with open (edges_file) as f:
            for lines in f:
                lines1=lines.split()

                if len(lines1) < 6:
                    continue

                G.add_edge(lines1[0] + "_" + lines1[3], lines1[1] + "_" + lines1[4],hinge_edge=int(lines1[5]))
                G.add_edge(lines1[1] + "_" + str(1-int(lines1[4])), lines1[0] + "_" + str(1-int(lines1[3])),hinge_edge=int(lines1[5]))


                if int(lines1[5]) != 0:

                    if int(lines1[5]) == 1: 

                        to_be_merged.append([lines1[1],lines1[6]])

                    elif int(lines1[5]) == -1:

                        to_be_merged.append([lines1[0],lines1[6]])

        for pair in to_be_merged:

            sink_node_long = hinge_mapping[pair[0]+'_0_'+pair[1]]
            sink_node = sink_node_long.split('_')[0]+'_'+sink_node_long.split('_')[1]

            if pair[0]+'_0' != sink_node:
                merge_a_to_b(G,pair[0]+'_0',sink_node)

            sink_node_long = hinge_mapping[pair[0]+'_1_'+pair[1]]
            sink_node = sink_node_long.split('_')[0]+'_'+sink_node_long.split('_')[1]

            if pair[0]+'_1' != sink_node:
                merge_a_to_b(G,pair[0]+'_1',sink_node)
            

    in_hinges = set()
    out_hinges = set()

    with open (hinge_file) as f:

        for lines in f:
            lines1=lines.split()
           
            if lines1[2] == '1':
                in_hinges.add(lines1[0]+'_0')
                out_hinges.add(lines1[0]+'_1')
            elif lines1[2] == '-1':
                in_hinges.add(lines1[0]+'_1')
                out_hinges.add(lines1[0]+'_0')


    json_file = open(gt_file)
    add_groundtruth(G,json_file,in_hinges,out_hinges)


    G0 = G.copy()

    nx.write_graphml(G0, prefix+'.'+'G0_merged'+'.graphml')


    G0s = random_condensation(G0,3500)

    nx.write_graphml(G0s, prefix+'.'+'G0s_merged'+'.graphml')


    G1=dead_end_clipping(G0,10)

    G1=z_clipping(G1,5,in_hinges,out_hinges)

    nx.write_graphml(G1, prefix+'.'+'G1_merged'+'.graphml')


    Gs = random_condensation(G1,2500)

    nx.write_graphml(Gs, prefix+'.'+'Gs_merged'+'.graphml')

  

if __name__ == "__main__":   

    read_graph(sys.argv[1],sys.argv[2],sys.argv[3],sys.argv[4])





